# --8<--8<--8<--8<--
#
# Copyright (C) 2006 Smithsonian Astrophysical Observatory
#
# This file is part of yaxx
#
# yaxx is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# yaxx is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#       Free Software Foundation, Inc. 
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--

package Yaxx;
use strict;
use warnings;

##****************************************************************************
# HERE are a handful of accessor methods which ensure that the values are
# properly initialized.  These may throw a fatal error if that is not
# possible.
##****************************************************************************

##****************************************************************************
sub XY {
##****************************************************************************
    my $self = shift;

    if (not defined $self->{X} or not defined $self->{Y}) {
	if (defined $self->{ra} and defined $self->{dec}) {
	    my ($out) = $self->DMcoord_coords( cel => ($self->{ra}, $self->{dec}));
	    $self->{X} = $out->{sky}{'x'};
	    $self->{Y} = $out->{sky}{'y'};
	} 

	# After all that, see if we have an X and Y!
	if (not defined $self->{X} or not defined $self->{Y}) {
	    message(1, "ERROR - could not determine source sky (X,Y).  Need\n");
	    message(1, "        (X,Y) or (ra,dec) in source table\n");
	    confess $Yaxx::FATAL_ERROR;
	}
    }

    return ($self->{X}, $self->{Y});
}

##****************************************************************************
sub ra_dec {
##****************************************************************************
    my $self = shift;
    my $coord = shift;

    if (not defined $self->{ra} or not defined $self->{dec}) {
	my ($out) = $self->DMcoord_coords( sky => $self->XY );
	$self->{ra} = $out->{cel}{deg}{ra};
	$self->{dec} = $out->{cel}{deg}{dec};

	if (not defined $self->{ra} or not defined $self->{dec}) {
	    message(1, "ERROR - could not determine source RA,Dec\n");
	    confess $Yaxx::FATAL_ERROR;
	}
    }

    return ($self->{ra}, $self->{dec});
}

##****************************************************************************
sub rad {
# Source extraction radius
##****************************************************************************
    my $self = shift;
    if (@_) {
	$self->{rad} = $_[0];
    }

    if (not defined $self->{rad}) {
	# No radius spec'd, so determine using the Chandra circular ECF values
	# determined by the cal group.  If an ECF object hasn't yet been
	# created, do so now
	if (not defined $Yaxx::ECF) {
	    $Yaxx::ECF = Chandra::Tools::ECF->new( $Chandra::Tools::ECF::Circular_File );
	}
	
	# Get telescope theta and phi (off-axis angle and azimuth) for source position
	my ($out) = $self->DMcoord_coords( sky => ($self->X, $self->Y) );
	my $theta_arcmin = $out->{theta} * 60;

	message(5, Dumper $out);

	if (not defined $out->{theta} or not defined $out->{phi}) {
	    message("ERROR - could not determine source theta and phi\n");
	    confess $Yaxx::FATAL_ERROR;
	}

	my %circ_ecf = $Yaxx::ECF->ECF(ecf    => $self->psf_fraction,
				 energy => $self->psf_energy,
				 theta  => $theta_arcmin,
				 phi    => $out->{phi},
				);
	$self->{rad} = sprintf("%.2f", $circ_ecf{radius} / $Yaxx::Arcsec_per_pixel);
	my $msg = sprintf("Extraction radius (%.0f%%) is %.2f pixels for (theta,phi)=(%.2f',%.1f deg)\n",
			  $self->psf_fraction * 100, $self->{rad}, $theta_arcmin, $out->{phi});
	message(3, $msg);

	if ($self->{rad} < $self->{min_src_rad}) {
	    $self->{rad} = $self->{min_src_rad};
	    message(3, "Expanding extraction radius to $self->{min_src_rad} pixels\n");
	}
    }

    return $self->{rad};
}

##****************************************************************************
sub ccdid {
##****************************************************************************
    my $self = shift;
    $self->{ccdid} = $_[0] if (@_);

    if (not defined $self->{ccdid}) {
	my ($out) = $self->DMcoord_coords( sky => ($self->X, $self->Y) );
	if (not defined ($self->{ccdid} = $out->{chip}{id})) {
	    message(1, "ERROR - could not determine source chip ID\n");
	    confess $Yaxx::FATAL_ERROR;
	}
    }

    return $self->{ccdid};
}

1;
